@@ -37,6 +37,7 @@ class Execution:
         self.anomalies = []
         self.plots = []
         self.result_dataframe = None
+        self.result_ds_dataframe = None
 
     def append_plot(self, figure, name="Unkwown"):
         plot = Plot()
@@ -90,6 +91,8 @@ class Execution:
             obj["anomaliesList"] = self.anomalies
         if self.result_dataframe is not None:
             obj["result"] = DataFrameToolkit.convert_to_json(self.result_dataframe)
+        if self.result_ds_dataframe is not None:
+            obj["result_ds_dataframe_id"] = str(self.result_ds_dataframe.df_id)
         return obj
 
     def from_json(self, obj):
